/* Copyright (c) 2006-2018 Peter Speck / Vitality. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Vitality nor the names of its contributors may be
 *       used to endorse or promote products derived from this software without
 *       specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY PETER SPECK / VITALITY AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL PETER SPECK / VITALITY AND CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR  SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.jabhts.junit.tests.pIterate;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;
import org.jabhts.annotation.element.EHtml;
import org.jabhts.annotation.element.EText;
import org.jabhts.annotation.flowControl.EIterate;
import org.jabhts.annotation.global.TCompile;
import org.jabhts.annotation.global.TOverrides;
import org.jabhts.junit.lib.TestBase;

@TCompile
public class TestIterate extends TestBase
{
	public TestIterate()
	{
		blockOneItems = new ArrayList<String>();
		for (int i = 1; i <= 5; i++)
			blockOneItems.add("scope-1-get #" + i);
		//
		blockTwoItems = new ArrayList<ComplexDataStructure>();
		for (int i = 1; i <= 3; i++)
			blockTwoItems.add(new ComplexDataStructure(i));
		//
		blockThreeItems = new TreeSet<Integer>();
		for (int i = 1; i <= 2; i++)
			blockThreeItems.add(i);
		//
		blockFourItems = new ArrayList<Map<String, Integer>>();
		for (int i = 1; i <= 3; i++)
			blockFourItems.add(new HashMap<String, Integer>());
	}

	@EText
	static String getTextOne()
	{
		return "Making a test";
	}

	@EText
	String getString1()
	{
		return "Outer-string-1";
	}

	@EHtml
	void printString2()
	{
		pw.htmlEncode("Outer-string-2");
	}

	//-----------------------------------------------

	@EIterate("BlockOne")
	List<String> blockOneItems;

	class BlockOne
	{
		final String msg;

		BlockOne(String msg)
		{
			this.msg = msg;
		}

		@EText
		@TOverrides
		String getString1()
		{
			return msg;
		}

		@EHtml
		@TOverrides
		void printString2()
		{
			pw.htmlEncode("str2-" + msg);
		}
	}

	//-----------------------------------------------
	class ComplexDataStructure
	{
		final int number;
		final String msg;

		ComplexDataStructure(int number)
		{
			this.number = number * (pw != null ? 1 : 42);
			this.msg = "scope-2-get #" + number;
		}
	}

	@EIterate("BlockTwo")
	Collection<ComplexDataStructure> blockTwoItems;

	BlockTwo newBlockTwo(ComplexDataStructure data) throws Exception
	{
		return new BlockTwo(data, true);
	}

	class BlockTwo
	{
		final ComplexDataStructure data;

		BlockTwo() throws Exception
		{
			throw new Exception("Should not be called");
		}

		@SuppressWarnings({"UnusedDeclaration"})
		BlockTwo(ComplexDataStructure data) throws Exception
		{
			throw new Exception("Should not be called");
		}

		BlockTwo(ComplexDataStructure data, boolean dummy) throws Exception
		{
			if (!dummy)
				throw new Exception("Bad mojo");
			this.data = data;
		}

		@EText
		@TOverrides
		String getString1()
		{
			return data.msg;
		}

		@EHtml
		@TOverrides
		void printString2()
		{
			pw.htmlEncode("str2-" + data.msg);
		}
	}

	//-----------------------------------------------

	Set<Integer> blockThreeItems;

	@EIterate
	Collection<Integer> iterBlockThree()
	{
		return blockThreeItems;
	}

	BlockThree newBlockThree(int x) throws Exception
	{
		return new BlockThree(x, 42);
	}

	class BlockThree
	{
		final int number;

		BlockThree(int number, int tt) throws Exception
		{
			this.number = number;
			if (tt != 42)
				throw new Exception("bad mojo");
		}

		@EText
		@TOverrides
		String getString1()
		{
			return "scope-3-get #" + number;
		}

		@EHtml
		@TOverrides
		void printString2()
		{
			pw.htmlEncode("scope-3-print-" + number);
		}
	}

	//-----------------------------------------------

	List<Map<String, Integer>> blockFourItems;

	@EIterate
	Collection<Map<String, Integer>> iterBlockFour()
	{
		return blockFourItems;
	}

	BlockFour newBlockFour(Map<String, Integer> x) throws Exception
	{
		x.put("num-" + (x.size() + 1), x.size() + 1);
		return new BlockFour(x, 42);
	}

	class BlockFour
	{
		final int number;

		BlockFour(Map<String, Integer> map, int tt) throws Exception
		{
			this.number = map.size();
			if (tt != 42)
				throw new Exception("bad mojo");
		}

		@EText
		@TOverrides
		String getString1()
		{
			return "scope-4-get #" + number;
		}

		@EHtml
		@TOverrides
		void printString2()
		{
			pw.htmlEncode("scope-4-print-" + number);
		}
	}

	//-----------------------------------------------

	@EHtml(tag = "title")
	void printTitle()
	{
		pw.htmlEncode("Testing gone amuck in iterating");
	}

	protected void emitTemplate() throws Exception
	{
		new TestIterateTp(this).emit(pw);
	}
}
