/* Copyright (c) 2006-2018 Peter Speck / Vitality. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Vitality nor the names of its contributors may be
 *       used to endorse or promote products derived from this software without
 *       specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY PETER SPECK / VITALITY AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL PETER SPECK / VITALITY AND CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR  SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.jabhts.translator.compiler.application.attribute;

import org.jabhts.translator.compiler.application.baseclass.Application;
import org.jabhts.translator.compiler.tree.AbstractEmitter;
import org.jabhts.translator.compiler.tree.ElementEmitters;
import org.jabhts.translator.compiler.utils.CompilerException;

public abstract class PostAttributeEmitter<AP extends Application> extends AbstractEmitter<AP>
{
	public final String[] lcAttributeNames;
	public boolean processed; // used by TreeEmit for keeping track of attribute collections.

	protected PostAttributeEmitter(ElementEmitters emitters, AP handler,
	                               String... lcAttributeNames)
	{
		super(emitters.program, handler);
		this.lcAttributeNames = lcAttributeNames;
		for (String attrName : lcAttributeNames) {
			if (emitters.alreadyDefined(emitters.ePostAttributes.get(attrName), this, "post-content attribute"))
				return;
			emitters.ePostAttributes.put(attrName, this);
		}
	}

	/**
	 * Is not allowed to emit quotes but only the content inside the quotes.
	 *
	 * @param staticAttrValue the attribute content specified in the html template
	 *                        which the method must prepend to the output. This is to enable proper use
	 *                        of ";" and " " separators in style and class attributes.
	 */
	public abstract void generatePostContent(String staticAttrValue) throws CompilerException;

}
