/* Copyright (c) 2006-2018 Peter Speck / Vitality. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Vitality nor the names of its contributors may be
 *       used to endorse or promote products derived from this software without
 *       specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY PETER SPECK / VITALITY AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL PETER SPECK / VITALITY AND CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR  SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.jabhts.translator.compiler.application.element.scoped;

import org.jabhts.annotation.flowControl.EScopeChilds;
import org.jabhts.annotation.flowControl.EScopingWhile;
import org.jabhts.translator.compiler.application.baseclass.AbstractBooleanApplication;
import org.jabhts.translator.compiler.application.element.ElementLoopEmitter;
import org.jabhts.translator.compiler.application.scope.Scope;
import org.jabhts.translator.compiler.application.scope.TargetingInfo;
import org.jabhts.translator.compiler.codeGeneration.api.PatchedGoto;
import org.jabhts.translator.compiler.codeGeneration.api.ProgramAddress;
import org.jabhts.translator.compiler.tree.ElementEmitters;
import org.jabhts.translator.compiler.utils.CompilerException;
import org.jabhts.translator.compiler.utils.LogLevel;
import org.w3c.dom.Element;

public class EScopingWhileApplication extends AbstractBooleanApplication
{
	public EScopingWhileApplication(TargetingInfo target) throws CompilerException
	{
		super(target);
	}

	public Scope makeSubScope(Scope outerScope) throws CompilerException
	{
		final EScopingWhile a = (EScopingWhile)annotation;
		Scope subScope = outerScope.getSubScope(a.scope(), target, null);
		if (subScope == null)
			return null; // we have already logged the error
		if (subScope.scopeClass.isAnnotationPresent(EScopeChilds.class)) {
			log.log(LogLevel.Error, "@EScope may not be present on inner-class used for @EScopingWhile");
			return null;
		}
		return subScope;
	}

	public void createEmitter(ElementEmitters emitters, Element e) throws CompilerException
	{
		final Scope subScope = makeSubScope(scope);
		if (subScope == null)
			return;
		new ElementLoopEmitter<AbstractBooleanApplication>(emitters, this, false, subScope)
		{
			PatchedGoto gotoTest;
			ProgramAddress bodyStart;

			public void preElement() throws CompilerException
			{
				gotoTest = program.addPatchedGoto();
				bodyStart = program.getNextAddress();
				program.addCreateScopeInstance(subScope);
			}

			public void postElement() throws CompilerException
			{
				program.addLeaveScopeInstance(subScope);
				gotoTest.patchToCurrentAddress();
				handler.emitBooleanCallback(program);
				final EScopingWhile a = (EScopingWhile)annotation;
				if (a.inverse())
					program.addNegateBoolean();
				program.addConditionalGoto(bodyStart);
			}
		};
	}
}