/* Copyright (c) 2006-2018 Peter Speck / Vitality. All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Vitality nor the names of its contributors may be
 *       used to endorse or promote products derived from this software without
 *       specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY PETER SPECK / VITALITY AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL PETER SPECK / VITALITY AND CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR  SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.jabhts.translator.compiler.htmlLoader;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import org.jabhts.translator.compiler.utils.CompilerUtils;
import org.jabhts.translator.globals.FileInfo;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

public class HtmlDocument
{
	public final FileInfo fileInfo;
	public final Document doc;
	public String docType;

	public HtmlDocument(File f, long moddate, Document doc, String docType)
	{
		fileInfo = new FileInfo(f, moddate);
		this.doc = doc;
		this.docType = docType;
	}

	public List<Element> findMatchingElements(String specification)
	{
		List<Element> list = new ArrayList<>();
		findMatchingElements(list, doc.getDocumentElement(), specification);
		return list;
	}

	private static void findMatchingElements(List<Element> list, Element e, String specification)
	{
		String id = e.getAttribute("id");
		if (specification.equalsIgnoreCase(id)) {
			list.add(e);
		} else {
			for (String s : CompilerUtils.splitCssClassNamesWithoutPrefixedUnderscore(e.getAttribute("class"))) {
				if (specification.equalsIgnoreCase(s)) {
					list.add(e);
					break;
				}
			}
		}
		for (Node child = e.getFirstChild(); child != null; child = child.getNextSibling()) {
			if (child.getNodeType() == Node.ELEMENT_NODE)
				findMatchingElements(list, (Element)child, specification);
		}
	}
}
