/* Copyright (C) 2008-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#import <Cocoa/Cocoa.h>
#import "TableViewHelper.h"
#import "VerticalViewFlow.h"

@class VerticalViewFlow;
@class Prefs;
@class AlertHelper;
@class ServerCommunication;
@class Filter;
@class FilterListView;
@class FilterPublisher;
@class FilterRuleTableView;

@interface FilterTabController : TableViewHelper <VerticalViewFlowDelegate,NSSplitViewDelegate,NSTableViewDelegate,NSTableViewDataSource> {
}

- (instancetype)initWithServerCommunication:(ServerCommunication*)comm
				withContainerView:(NSView*)containerView
					  withTabView:(NSTabView*)tabView
		   withFiltersTabViewItem:(NSTabViewItem*)filtersTabViewItem
					   withBundle:(NSBundle*)bundle;
- (void)postInit;
- (void)dealloc;

@property (readonly, copy) NSImage *subscriptionGlobeImage;
@property (readonly, copy) NSImage *publishGlobeImage;
@property (readonly, copy) NSImage *publishGlobeDirtyImage;
@property (readonly, copy) NSImage *warningImage;
@property (readonly, copy) NSImage *updateAvailableImage;

@property (readonly) Filter *createNewUntitledFilter;

- (IBAction)filterAddAction:(id)sender;
- (IBAction)filterRemoveAction:(id)sender;
- (IBAction)filterEditAction:(id)sender;
- (IBAction)filterActionAction:(id)sender;
//
- (IBAction)filterActionDuplicateAction:(id)sender;
//
- (IBAction)filterCommentsAction:(id)sender;
//
- (IBAction)ruleAddAction:(id)sender;
- (IBAction)ruleRemoveAction:(id)sender;
- (IBAction)ruleActionAction:(id)sender;
- (IBAction)ruleActionDuplicateAction:(id)sender;
- (void)setSavedRuleSelection:(NSArray*)ruleList;

- (void)ruleEditCommit:(NSXMLElement*)filterE
			withFilter:(Filter*)filter
			  isVirgin:(BOOL)virgin;

@property (readonly) Filter *selectedFilter;
- (Filter*)selectFilterById:(NSInteger)filterId;
- (Filter*)getFilterById:(NSInteger)filterId;


// Add subscription sheet:
- (IBAction)filterActionSubscribeAction:(id)sender;
- (IBAction)subAddConfirmAction:(id)sender;
- (IBAction)subAddCancelAction:(id)sender;

// Update subscriptions sheet:
- (IBAction)updateSelectedSubscriptionAction:(id)sender;
- (IBAction)updateAllSubscriptionsAction:(id)sender;
- (void)notifyUpdatedFilter:(Filter*)filter;

// Publish
- (IBAction)publishWebDavAction:(id)sender;

// Utils
@property (readonly) NSBundle *bundle;
@property (readonly) Prefs *prefs;
@property (readonly) AlertHelper *alertHelper;
@property (readonly) ServerCommunication *comm;
@property (weak, readonly) FilterRuleTableView *rulesTableView;

- (NSArray*)filterArray;
- (NSArray*)ruleElements;
- (NSBundle*)mainBundle;

- (void)updateTabViewWithForceUpdate:(BOOL)forceUpdate;
- (void)redrawRulesTable;
- (void)notifyUpdatedAuthorizationState;

@end
