/* Copyright (C) 2009-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

//
//  NamedEnum.m
//  GlimmerBlockerPrefsPane
//
//  Created by Peter Speck on 21/05/09.
//  Copyright 2009 Vitality. All rights reserved.
//

#import "NamedEnum.h"
#import "Xml.h"
#import "Prefs.h"

@interface NamedEnum()

@property NSMutableArray* valueArray;
@property NSUInteger defaultValueIndex;

@end

@implementation NamedEnum

- (instancetype)initWithSemicolonList:(NSString*)encoded
{
	if (!(self = [super init]))
		return nil;
	NSArray* a = [encoded componentsSeparatedByString:@";"];
	NSInteger num = [a count];
	_valueArray = [NSMutableArray arrayWithCapacity:num];
	_defaultValueIndex = NSNotFound;
	for (int i = 0; i < num; i++) {
		NSString* s = a[i];
		if ([s hasPrefix:@"+"]) {
			_defaultValueIndex = i;
			s = [s substringFromIndex:1];
		}
		[_valueArray addObject:s];
	}
	if (_defaultValueIndex == NSNotFound) {
		NSLog(@"NamedEnum: Default value missing in semicolon list '%@'", encoded);
		_defaultValueIndex = 0;
	}
	return self;
}


- (NSString*)nameFromIndex:(NSInteger)idx
{
	if (idx >= 0 && idx <= (NSInteger)[_valueArray count])
		return _valueArray[idx];
	else
		return _valueArray[_defaultValueIndex];
}

- (NSInteger)indexFromXPath:(NSString*)xpath
				  withPrefs:(Prefs*)prefs
{
	return [self indexFromXPath:xpath withPrefs:prefs withDefault:NULL];
}

- (NSInteger)indexFromXPath:(NSString*)xpath
				  withPrefs:(Prefs*)prefs
				withDefault:(NSString*)defaultValue
{
	NSString* s = [prefs stringForXPath:xpath];
	NSString* value = [s length] ? s : defaultValue;
	NSUInteger idx = value ? [_valueArray indexOfObject:value] : NSNotFound;
	//DebugNSLog(@"NamedEnum.update: %@ = '%@'/'%@' = %d  values %@", xpath, s, value, idx, _valueArray);
	if (idx == NSNotFound)
		idx = _defaultValueIndex;
	return idx;
}

- (NSInteger)indexFromXPath:(NSString*)xpath
				   withRoot:(NSXMLElement*)rootE
				withDefault:(NSString*)defaultValue
{
	NSString* s = [Xml stringForXPath:xpath withRoot:rootE];
	NSString* value = [s length] ? s : defaultValue;
	NSUInteger idx = value ? [_valueArray indexOfObject:value] : NSNotFound;
	//DebugNSLog(@"NamedEnum.update: %@ = '%@'/'%@' = %d  values %@", xpath, s, value, idx, _valueArray);
	if (idx == NSNotFound)
		idx = _defaultValueIndex;
	return idx;
}


- (void)selectDefaultItemInPop:(NSPopUpButton*)pop
{
	[pop selectItemAtIndex:_defaultValueIndex];
}

- (void)updatePopButton:(NSPopUpButton*)pop
				toValue:(NSString*)value
{
	NSUInteger idx = value ? [_valueArray indexOfObject:value] : NSNotFound;
	//DebugNSLog(@"NamedEnum.update: '%@' = %d  values %@", value, idx, _valueArray);
	if (idx == NSNotFound)
		idx = _defaultValueIndex;
	[pop selectItemAtIndex:idx];
}

- (void)updatePopButton:(NSPopUpButton*)pop
			  fromXPath:(NSString*)xpath
			   withRoot:(NSXMLElement*)rootE
{
	[self updatePopButton:pop fromXPath:xpath withRoot:rootE withDefault:NULL];
}

- (void)updatePopButton:(NSPopUpButton*)pop
			  fromXPath:(NSString*)xpath
			   withRoot:(NSXMLElement*)rootE
			withDefault:(NSString*)defaultValue
{
	NSInteger idx = [self indexFromXPath:xpath withRoot:rootE withDefault:defaultValue];
	[pop selectItemAtIndex:idx];
}

- (NSString*)selectedPopValue:(NSPopUpButton*)pop
{
	NSInteger idx = [pop indexOfSelectedItem];
	if (idx < 0)
		idx = _defaultValueIndex;
	if (idx >= (int)[_valueArray count]) {
		NSLog(@"NamedEnum: Selected-index = %ld => too few items in value array: %@", (long)idx, _valueArray);
		return NULL;
	}
	NSString* s = _valueArray[idx];
	return [s isEqual:@"ignored"] ? NULL : s;
}

@end
