/* Copyright (C) 2008-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#import <Cocoa/Cocoa.h>

@class VerticalViewFlow;

@protocol VerticalViewFlowDelegate
@optional
- (void)vertPatchTableAddedRow:(id)sender;
- (void)vertPatchTableRemovedRow:(id)sender;
- (void)vertPatchTableChangedSelection:(id)sender;
- (BOOL)respondsToSelector:(SEL)aSelector;
@end

@interface VerticalViewFlowRow : NSView
{
@private
	BOOL _selected;
	NSUInteger _rowNumber;
}

- (instancetype)initWithFrame:(NSRect)frameRect;

- (NSUInteger)rowNumber;
- (BOOL)selected;
- (BOOL)shouldDrawUsingFocusColor;

- (BOOL)isEditing;
- (void)endEditing;

- (VerticalViewFlow*)tableView;


// private
//- (void)setSelected:(BOOL)selected;
//- (void)setRowNumber:(NSUInteger)rowNumber;

@end


@interface VerticalViewFlow : NSView
{
	@private
	NSMutableArray* _rows;
	BOOL _freezeUpdates;

}
@property id<VerticalViewFlowDelegate> delegate;
@property (readonly) BOOL isFirstResponder;

- (instancetype)initWithFrame:(NSRect)frameRect;
- (void)dealloc;

- (void)setDelegate:(id<VerticalViewFlowDelegate>)delegate;
- (id<VerticalViewFlowDelegate>)delegate;

- (void)addRow:(VerticalViewFlowRow*)view atIndex:(NSUInteger)idx;
- (void)removeRow:(VerticalViewFlowRow*)view;

- (id)rowByIndex:(NSUInteger)idx;
- (NSUInteger)rowCount;
- (NSArray*)rows;

- (void)setFreezeUpdates:(BOOL)freeze;
- (void)updateRowViewFrames;
- (void)repaintSelectedRows;

- (void)setSelectionByView:(VerticalViewFlowRow*)selectedView;
- (void)setSelectionByIndex:(NSUInteger)idx;
- (id)firstSelectedRow;
- (id)lastSelectedRow;

- (BOOL)acceptsFirstResponder;
- (void)keyDown:(NSEvent*)theEvent;

- (BOOL)isFirstResponder;

@end

@interface ScrollViewWithFocusRing : NSScrollView
{
	BOOL _drawFocusRing;
}
- (instancetype)initWithFrame:(NSRect)frameRect;

- (void)setDrawFocusRing:(BOOL)flag;

- (void)drawRect:(NSRect)r;

@end
