/* Copyright (C) 2008-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.glimmerblocker.proxy.proxyRequest.keyword;

import dk.vitality.lib.foundation.exception.ExceptionUtils;

/**
 * rfc 3492
 */
public class Punycode
{
	static final int TMIN = 1;
	static final int TMAX = 26;
	static final int BASE = 36;
	static final int INITIAL_N = 128;
	static final int INITIAL_BIAS = 72;
	static final int DAMP = 700;
	static final int SKEW = 38;

	public static void main(String[] args)
	{
		for (String s : args) {
			System.err.println("Input:   '" + s + "'");
			try {
				if (s.startsWith("xn--"))
					s = s.substring(4);
				System.err.println("Decoded: '" + decode(s) + "'");
			}
			catch (Exception ex) {
				System.err.println("Failed decoding: " + ExceptionUtils.getExMsg(ex));
				ex.printStackTrace();
			}
		}
	}

	static String decode(String s) throws PunycodeException
	{
		StringBuilder sb = new StringBuilder(s.length());
		int d = s.lastIndexOf('-');
		if (d <= 0) {
			d = 0;
		} else {
			for (int i = 0; i < d; i++) {
				char ch = s.charAt(i);
				if (ch >= 128)
					throw new PunycodeException();
				sb.append(ch);
			}
			d++;
		}
		int n = INITIAL_N;
		int i = 0;
		int bias = INITIAL_BIAS;
		while (d < s.length()) {
			int prev_i = i;
			int w = 1;
			for (int k = BASE; ; k += BASE) {
				if (d == s.length())
					throw new PunycodeException();
				int ch = s.charAt(d++);
				int digit = codepoint2digit(ch);
				if (digit > (Integer.MAX_VALUE - i) / w)
					throw new PunycodeException();
				i = i + digit * w;
				int t;
				if (k <= bias)
					t = TMIN;
				else if (k >= bias + TMAX)
					t = TMAX;
				else
					t = k - bias;
				if (digit < t)
					break;
				w *= (BASE - t);
			}
			bias = adapt(i - prev_i, sb.length() + 1, prev_i == 0);
			if (i / (sb.length() + 1) > Integer.MAX_VALUE - n)
				throw new PunycodeException();
			n += i / (sb.length() + 1);
			i %= (sb.length() + 1);
			sb.insert(i, (char)n);
			i++;
		}
		return sb.toString();
	}

	private static int adapt(int delta, int numpoints, boolean first)
	{
		if (first)
			delta /= DAMP;
		else
			delta /= 2;
		delta += (delta / numpoints);
		int k = 0;
		while (delta > ((BASE - TMIN) * TMAX) / 2) {
			delta /= (BASE - TMIN);
			k += BASE;
		}
		return k + ((BASE - TMIN + 1) * delta) / (delta + SKEW);
	}

	private static int codepoint2digit(int ch) throws PunycodeException
	{
		if (ch >= '0' && ch <= '9')
			return ch - '0' + 26;
		if (ch >= 'a' && ch <= 'z')
			return ch - 'a';
		if (ch >= 'A' && ch <= 'Z')
			return ch - 'A';
		throw new PunycodeException();
	}

	static class PunycodeException extends Exception
	{
	}
}
