/* Copyright (C) 2008-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.glimmerblocker.proxy.rhino.runtime;

import org.mozilla.javascript.ClassShutter;
import org.mozilla.javascript.Context;
import org.mozilla.javascript.ContextFactory;

class GbContextFactory extends ContextFactory
{
	protected Context makeContext()
	{
		MyContext ctx = new MyContext(this);
		// Use pure interpreter mode to allow for
		// observeInstructionCount(Context, int) to work
		ctx.setOptimizationLevel(-1);
		// Make Rhino runtime to call observeInstructionCount
		// each x bytecode instructions.
		// 10000 is less than 1 msecs on Core 2 Duo.
		ctx.setInstructionObserverThreshold(10 * 1000 * 1000); // ~200 msecs.
		ctx.setLanguageVersion(Context.VERSION_1_7);
		ctx.setClassShutter(new ClassShutter()
		{
			public boolean visibleToScripts(String s)
			{
				/**
				 * LiveConnect is not included in the smalljs.jar
				 * version of Rhine, but be really paranoid and explicit disable
				 * all access:
				 */
				return false;
			}
		});
		return ctx;
	}

	protected void observeInstructionCount(Context cx, int instructionCount)
	{
		MyContext mcx = (MyContext)cx;
		long currentTime = System.currentTimeMillis();
		long dur = currentTime - mcx.startTime;
		if (dur > 5 * 1000)
			throw new ScriptTimeoutError("Script runs for too long time. Aborted.");
	}

	private static class MyContext extends Context
	{
		long startTime = System.currentTimeMillis();

		MyContext(ContextFactory contextFactory)
		{
			// Rhino 1.7R2pre needed for protected super constructor with factory arg.
			super(contextFactory);
		}
	}
}
