/* Copyright (C) 2008-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.glimmerblocker.proxy.rule;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public enum KeywordMatchType
{
	// least specific first, and most specific last
	Regexp() {
		@NotNull
		public String debugDescription(@Nullable String value)
		{
			return "regexp{" + value + "}";
		}

		@NotNull
		public String userDescription(@Nullable String value)
		{
			return "query matches regexp: " + value;
		}
	},
	StartsWith() {
		@NotNull
		public String debugDescription(@Nullable String value)
		{
			return "starts-with{" + value + "}";
		}

		@NotNull
		public String userDescription(@Nullable String value)
		{
			return "keyword starts with: " + value;
		}
	},
	Is() {
		@NotNull
		public String debugDescription(@Nullable String value)
		{
			return "is{" + value + "}";
		}

		@NotNull
		public String userDescription(@Nullable String value)
		{
			return "keyword is: " + value;
		}
	};

	@NotNull
	public abstract String debugDescription(@Nullable String value);

	@NotNull
	public abstract String userDescription(@Nullable String value);

	public static KeywordMatchType fromXmlValue(@Nullable String v) throws IllegalArgumentException
	{
		if (v == null || v.equals("is"))
			return Is;
		if (v.equals("regexp"))
			return Regexp;
		if (v.equals("starts-with"))
			return StartsWith;
		throw new IllegalArgumentException("Invalid keyword-type='" + v + "'");
	}
}
