/* Copyright (C) 2008-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.glimmerblocker.proxy.rule;

import java.util.regex.Pattern;
import org.glimmerblocker.proxy.rhino.runtime.RhinoRuntimeCache;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import dk.vitality.lib.foundation.xml.xpath.XmlArgs;
import static java.util.regex.Pattern.CASE_INSENSITIVE;

public class KeywordRule extends AbstractRule
{
	@NotNull public final String keyword;
	@NotNull public final KeywordMatchType keywordMatchType;

	@Nullable public final Pattern keywordRegexp;

	public final boolean useJavascript;
	@Nullable public final String urlTemplate;
	@Nullable public final String javascript;
	public final int runtimeId;

	public KeywordRule(boolean isPreview,
	                   int filterId,
	                   @NotNull String filterName,
	                   @NotNull XmlArgs ruleXA) throws Exception
	{
		super(isPreview, filterId, filterName, ruleXA, RuleType.Keyword);
		//
		this.keywordMatchType = KeywordMatchType.fromXmlValue(ruleXA.getOptString("@keyword-type"));
		this.keyword = ruleXA.getReqString("@keyword");
		this.keywordRegexp = (keywordMatchType == KeywordMatchType.Regexp)
				? Pattern.compile(keyword, CASE_INSENSITIVE) : null;
		//
		XmlArgs keywordXA = ruleXA.getReqXmlArgs("keyword");
		useJavascript = keywordXA.getOptBoolean("@keyword-uses-js");
		if (useJavascript) {
			this.urlTemplate = null;
			this.javascript = keywordXA.getReqString(".");
			this.runtimeId = RhinoRuntimeCache.makeNewRuntimeId();
		} else {
			this.urlTemplate = keywordXA.getReqString("@keyword-url");
			this.javascript = null;
			this.runtimeId = 0;
		}
	}

	/**
	 * Do NOT rely on the contents of this string for anything. It is only for debugging.
	 */
	public String toString()
	{
		StringBuilder sb = new StringBuilder(100);
		sb.append("Keyword");
		sb.append("{");
		sb.append(keywordMatchType.debugDescription(keyword));
		sb.append(";");
		toStringMatching(sb);
		sb.append("}");
		appendFilterName(sb);
		return sb.toString();
	}
}
