/* Copyright (C) 2008-2015 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.glimmerblocker.proxy.util;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import org.jetbrains.annotations.NotNull;
import dk.vitality.lib.foundation.exception.ExceptionUtils;
import dk.vitality.lib.foundation.io.SimpleUnixExec;

public class ResourceUtils
{
	@NotNull
	public static byte[] getResource(@NotNull String path) throws IOException
	{
		return getResource(path, ResourceUtils.class);
	}

	@NotNull
	public static byte[] getResource(@NotNull String path, Class<?> cls) throws IOException
	{
		InputStream is = cls.getResourceAsStream(path);
		if (is == null) {
			System.err.println("Missing resource: '" + path + "'");
			throw new FileNotFoundException(path);
		}
		try {
			ByteArrayOutputStream baos = new ByteArrayOutputStream(is.available());
			int x;
			while ((x = is.read()) >= 0)
				baos.write(x);
			return baos.toByteArray();
		}
		finally {
			is.close();
		}
	}

	public static void setPermissions(File f, String chmodPerms)
	{
		try {
			String[] cmd = {"chmod", chmodPerms, f.getPath()};
			SimpleUnixExec sue = new SimpleUnixExec(cmd);
			sue.run();
			int status = sue.getResultCode();
			if (status != 0)
				throw new Exception("exit-code = " + status);
		}
		catch (Exception ex) {
			System.err.println("Failed changing permissions: " + ExceptionUtils.getExMsg(ex));
		}
	}
}
