#!/bin/bash

# OBS:  Does NOT build java and assumes java is already compiled into *.class files.
set -e
#set -v

echo "Needs sudo active:"
sudo date

CLEAN=${CLEAN:+1}
XCODEBUILD=${XCODEBUILD:-xcodebuild}

if [ "$DEBUG" != "" ]
then
	CONFIGURATION=Debug
else
	CONFIGURATION=Release
fi


cd "`dirname "$0"`"
cd ..



pushd ..
export SYMROOT=`pwd`/tmp/xcode
popd

XCODEBUILD="$XCODEBUILD -configuration $CONFIGURATION SYMROOT=$SYMROOT"

if [ "$CLEAN" != "" ]
then
	echo "####### Cleans build"
	rm -rf ./Build ./*/[bB]uild/*/*
	rm -rf /tmp/*.dst # Xcode creates those.
	rm -rf "$SYMROOT"
	for dir in AuthHelper Installer NetworkConfigurationListener NetworkSettingsUpdater PreferencePane ProxyApp
	do
		cd $dir
		$XCODEBUILD -alltargets clean 2>&1 | fgrep '=='
		cd ..
	done
fi

JVMSRC=./JVM/jre-bundle
if [ ! -e $JVMSRC/Contents/Home/jre/lib ]
then
	echo "MISSING JVMSRC dir: $JVMSRC"
	echo "Making JRE bundle, takes some time."
	./JVM/make-jre-bundle.sh
	echo "Done making JRE bundle."
fi

echo "####### builds prefpane"
mkdir -p $SYMROOT

# always direct 'install' option without building first, or Xcode will do something way different.
echo $XCODEBUILD -verbose -target Everything
$XCODEBUILD -verbose -target Everything


# https://developer.apple.com/library/mac/technotes/tn2206/_index.html
# Code signing uses extended attributes to store signatures in non-Mach-O executables such as script files.
# If the extended attributes are lost then the program's identity will be broken.
# Thus, when you ship your script, you must use a mechanism that preserves extended attributes.
#

if [ "$MAKERELEASE" ]
then
	DST=../tmp/GlimmerBlocker.prefPane
	echo "####### installs in ../tmp/GlimmerBlocker.prefPane"
else
	DST=/Library/PreferencePanes/GlimmerBlocker.prefPane
	echo "####### installs in /Library"
fi

RSYNC="rsync --extended-attributes --ignore-times --checksum --archive --no-owner --no-group --delete-after"

sudo mkdir -p $DST
sudo $RSYNC \
	--exclude='*.app' \
	$SYMROOT/$CONFIGURATION/GlimmerBlocker.prefPane/ \
	$DST

for app in $SYMROOT/$CONFIGURATION/*.app
do
	sudo $RSYNC \
		--delete-after \
		--exclude=jre \
		--exclude=Java \
		$app \
		$DST/Contents/
done

# Copy JRE directly as it's way big for intermediates.
JVMDST=$DST/Contents/GlimmerBlockerProxy.app/Contents/Resources/jre
echo JVMSRC: $JVMSRC
#codesign -v --verbose=9 $JVMSRC

echo JVMDST: $JVMDST
sudo mkdir -p $JVMDST
sudo $RSYNC $JVMSRC/ $JVMDST
#codesign -v --verbose=9 $JVMDST
# When GlimmerBlockerProxy.app is signed, the JRE plugin on its own is no longer valid, only as part of GlimmerBlockerProxy.app
echo

export JARS_PATH=$DST/Contents/GlimmerBlockerProxy.app/Contents/Resources/Java
sudo mkdir -p $JARS_PATH
./Proxy/install-jars.sh

if [ "$MAKERELEASE" ]
then
	for f in $DST/Contents/*.app $DST
	do
		echo
		echo "Signs `basename $f`"
		sudo ./bin/codesign.pl "$f"
		echo "Checks codesign"
		codesign -v "$f"
		echo
	done
else
	find $DST \! -name AuthHelper -print0 | xargs -0 sudo chown _glimmerblocker:_glimmerblocker
	find $DST -name AuthHelper -print0 | xargs -0 sudo chown root:admin
	find $DST -name AuthHelper -print0 | xargs -0 sudo chmod 6555
fi
echo "=== Building done."

