/* Copyright (C) 2008-2012 Peter Speck
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#import <Cocoa/Cocoa.h>

@interface XmlDoc : NSObject {

@private
	NSURL *_url;
	NSError* _error;
	NSXMLDocument* _doc;
	NSXMLElement* _rootE;
}

- (instancetype)initWithRootElement:(NSXMLElement*)rootE;
- (instancetype)initWithRootElementTagName:(NSString*)tagName;
- (instancetype)initWithURL:(NSURL*)url withData:(NSData*)data;
- (void)dealloc;
- (void)reloadWithRootElement:(NSXMLElement*)rootE;

- (NSURL*) url;
- (NSError*) error;
- (NSXMLDocument*) doc;
- (NSXMLElement*) rootE;
- (NSXMLElement*) byXPath:(NSString*)xpath;

- (NSData*)printToData;
- (NSString*)printToString;

- (NSXMLElement*)createRootChildWithName:(NSString*)name;

//-------------------------------------------------------------- XPath scalars
// xpaths for these methods *must* be of the form
//  path/to/element/@attrname
- (NSString*)stringForXPath:(NSString*)xpath;
- (void)setString:(NSString*)value forXPath:(NSString*)xpath;

- (double)doubleForXPath:(NSString*)xpath;
- (void)setDouble:(double)value forXPath:(NSString*)xpath;

- (int)intForXPath:(NSString*)xpath;
- (void)setInt:(int)value forXPath:(NSString*)xpath;

- (BOOL)boolForXPath:(NSString*)xpath;
- (void)setBool:(BOOL)value forXPath:(NSString*)xpath;

- (NSString*)dateFromMillisecsXPath:(NSString*)xpath;

#pragma mark -------------------------------- XPath

- (NSArray*)nodesForXPath:(NSString *)xpath;
- (NSXMLNode*)nodeForXPath:(NSString *)xpath;
- (void)removeElementsForXPath:(NSString*)xpath;

- (NSXMLElement*)singletonElementForXPath:(NSString*)xpath;

- (NSXMLElement*)singletonElementForXPath:(NSString*)xpath
							 asFirstChild:(BOOL)asFirstChild;

#pragma mark -------------------------------- unqiue id

- (int)makeUniqueId;

@end
